/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2008  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS version: 3.80f                                         *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for M68DEMO908GB60
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"
#include "RTOS.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/****** Addresses of sfrs and peripheral identifiers ****************/
#define PTFD    (*(volatile OS_U8*) (0x0040))  // Port F Data Register
#define PTFDD   (*(volatile OS_U8*) (0x0043))  // Port F Data Direction Register
#define PTFPE   (*(volatile OS_U8*) (0x0041))  // Port F Pull Enable Register

/****** Assign LEDs to Ports ****************************************/
#define LED0  (1 << 0)
#define LED1  (1 << 1)
#define LED_ALL (LED0 | LED1)

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  PTFDD  = LED_ALL;
  PTFD  &= ~LED_ALL;
}

/*********************************************************************
*
*       LED switching routines
*       LEDs are switched on with low level on port lines
*/
void BSP_SetLED(int Index) {
  PTFD &= ~(1 << Index);           
}

void BSP_ClrLED(int Index) {
  PTFD |= (1 << Index);           
}

void BSP_ToggleLED(int Index) {
  if (PTFD & (1 << Index)) {
    PTFD &= ~(1 << Index);           
  } else {
    PTFD |= (1 << Index);           
  }
}

/****** EOF *********************************************************/


