/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2008  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS version: 3.80f                                         *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSINIT_MC9S08GB60A.c
          for Freescale SO8 (MC9S08GB60A)

Purpose : Initializes and handles the hardware for the OS as far
          as required by the OS.
          Feel free to modify this file acc. to your
          target system.

--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

#define ALLOW_NESTED_INTERRUPTS  0  // Caution: Nesting interrupts will cause higher stack load on system stack CSTACK

/*********************************************************************
*
*       Clock frequency settings
*/
#define OS_FSYS (39990857L)

#ifndef   OS_TICK_FREQ
  #define OS_TICK_FREQ (1000)
#endif

#ifndef   OS_PCLK_TIMER
  #define OS_PCLK_TIMER  (OS_FSYS/2)
#endif

/*********************************************************************
*
*       Configuration of communication to OSView
*/
#ifndef   OS_VIEW_ENABLE            // Global enable of communication
  #define OS_VIEW_ENABLE    (1)     // Default: on
#endif

/*********************************************************************
*
*       UART settings for OSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to OSView, please define it to be -1
*       Currently UART0 and UART1 are supported and the standard
*       setup enables UART 0 per default
*/
#ifndef   OS_UART
  #define OS_UART (0)
#endif

#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE (38400)
#endif

#ifndef   OS_PCLK_UART
  #define OS_PCLK_UART  (OS_FSYS/2)
#endif

/****** End of configuration settings *******************************/

/*********************************************************************
*
*       UART settings for OSView
*
*       Automatically generated from configuration settings
*/
#define OS_UART_USED   (OS_VIEW_ENABLE && ((OS_UART == 0) || (OS_UART == 1)))

/*********************************************************************
*
*       Local defines (sfrs used in RTOSInit.c)
*
**********************************************************************
*/
#define SOPT       (*(volatile OS_U8*) (0x1802))  // System Options Register

#define TPM1SC     (*(volatile OS_U8*) (0x0030))  // Status and Control Register
#define TPM1CNT    (*(volatile OS_U16*)(0x0031))  // Counter Register 
#define TPM1MOD    (*(volatile OS_U16*)(0x0033))  // Timer Counter Modulo Register
#define TPM1C0SC   (*(volatile OS_U8*) (0x0035))  // Timer Channel 0 Status and Control Register 
#define TPM1C0V    (*(volatile OS_U16*)(0x0036))  // Timer Channel 0 Value Register

#define SCI1BDH    (*(volatile OS_U8*) (0x0018))  // Baud Rate Register High 
#define SCI1BDL    (*(volatile OS_U8*) (0x0019))  // Baud Rate Register Low
#define SCI1C1     (*(volatile OS_U8*) (0x001A))  // Control Register 1
#define SCI1C2     (*(volatile OS_U8*) (0x001B))  // Control Register 1
#define SCI1S1     (*(volatile OS_U8*) (0x001C))  // Status Register 1
#define SCI1S2     (*(volatile OS_U8*) (0x001D))  // Status Register 2
#define SCI1C3     (*(volatile OS_U8*) (0x001E))  // Control Register 3
#define SCI1D      (*(volatile OS_U8*) (0x001F))  // Data Register

#define SCI2BDH    (*(volatile OS_U8*) (0x0020))  // Baud Rate Register High 
#define SCI2BDL    (*(volatile OS_U8*) (0x0021))  // Baud Rate Register Low
#define SCI2C1     (*(volatile OS_U8*) (0x0022))  // Control Register 1
#define SCI2C2     (*(volatile OS_U8*) (0x0023))  // Control Register 1
#define SCI2S1     (*(volatile OS_U8*) (0x0024))  // Status Register 1
#define SCI2S2     (*(volatile OS_U8*) (0x0025))  // Status Register 2
#define SCI2C3     (*(volatile OS_U8*) (0x0026))  // Control Register 3
#define SCI2D      (*(volatile OS_U8*) (0x0027))  // Data Register

#define ICGC1      (*(volatile OS_U8*) (0x0048))  // ICG Control Register 1
#define ICGC2      (*(volatile OS_U8*) (0x0049))  // ICG Control Register 2
#define ICGS1      (*(volatile OS_U8*) (0x004A))  // ICG Status Register 1
#define ICGTRM     (*(volatile OS_U8*) (0x004E))  // ICG Trim value in RAM
#define TRIM_VALUE (*(volatile OS_U8*) (0xFFBE))  // ICG Trim value in Flash

#if (OS_UART == 0) 
  #define SCIxBDH    SCI1BDH 
  #define SCIxBDL    SCI1BDL 
  #define SCIxC1     SCI1C1
  #define SCIxC2     SCI1C2      
  #define SCIxS1     SCI1S1 
  #define SCIxS2     SCI1S2
  #define SCIxC3     SCI1C3
  #define SCIxD      SCI1D 
#elif (OS_UART == 1) 
  #define SCIxBDH    SCI2BDH 
  #define SCIxBDL    SCI2BDL 
  #define SCIxC1     SCI2C1
  #define SCIxC2     SCI2C2      
  #define SCIxS1     SCI2S1 
  #define SCIxS2     SCI2S2
  #define SCIxC3     SCI2C3
  #define SCIxD      SCI2D 
#endif

#define TPM1C0SC_CH0F            (1 << 7)

#define SCIxS1_ERRORS_MASK       0x0F  
#define SCIxS1_RDRF              (1 << 5)
#define SCIxS1_TDRE              (1 << 7)
#define SCIxC2_TE                (1 << 3)
#define SCIxC2_RE                (1 << 2)
#define SCIxC2_RIE               (1 << 5)
#define SCIxC2_TIE               (1 << 6)
         
/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*       Initialize the hardware (timer) required for the OS to run.
*       May be modified, if an other timer should be used
*/
#define OS_TIMER_RELOAD (OS_PCLK_TIMER/OS_TICK_FREQ)
#if (OS_TIMER_RELOAD >= 0x10000)
  #error "TPM1 timer can not be used, please check configuration"
#endif

void OS_InitHW(void) {
  OS_IncDI();
  //
  //  Disable Watchdog
  //
  SOPT &= ~0x80;
  //
  // Init PLL, use internal RC oscillator
  //
  ICGTRM = TRIM_VALUE;           // Needs calibration from trim value in flash
  ICGC1 = 0x4C;                 
  ICGC2 = 0x70;                 
  while (ICGS1 & (1 << 3));
  //
  // Initialize embOS timer
  //
  TPM1SC   = 0x00;               // Stop HW; disable overflow interrupt and set prescaler to 0 
  TPM1C0SC = 0x50;               // Set output compare mode and enable compare interrupt 
  TPM1C0V  = (OS_TIMER_RELOAD);  // Initialize appropriate value to the compare/modulo/reload register      
  TPM1MOD  = (OS_TIMER_RELOAD);
  TPM1CNT  = 0x00;               // Reset HW Counter  
  TPM1SC   = 0x08;               // No prescaler, select BusClk and run timer  
  OS_COM_Init();
  OS_DecRI();
}

/*********************************************************************
*
*       Idle loop  (OS_Idle)
*
*       Please note:
*       This is basically the "core" of the idle loop.
*       This core loop can be changed, but:
*       The idle loop does not have a stack of its own, therefore no
*       functionality should be implemented that relies on the stack
*       to be preserved. However, a simple program loop can be programmed
*       (like toggeling an output or incrementing a counter)
*/
void OS_Idle(void) {           // Idle loop: No task is ready to exec 
  while (1) {
  }
}
  
/*********************************************************************
*
*        _OS_SystemIrqhandler
*/
__interrupt void _OS_SystemIrqhandler(void) {
  TPM1C0SC &= ~TPM1C0SC_CH0F;  // Reset compare interrupt request flag, TPM1C0SC: CH0F=0  
#if (ALLOW_NESTED_INTERRUPTS == 1)
  OS_EnterNestableInterrupt();
#else
  OS_EnterInterrupt();
#endif  
  OS_TICK_Handle();  
#if (ALLOW_NESTED_INTERRUPTS == 1)
  OS_LeaveNestableInterrupt();
#else
  OS_LeaveInterrupt();
#endif  
}
  
/*********************************************************************
*
*       Get time [cycles]
*
*       This routine is required for task-info via OSView or high
*       resolution time maesurement functions.
*       It returns the system time in timer clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  OS_U32 Time;
  OS_U32 Cnt;
  
  Time = OS_GetTime32();
  Cnt  = TPM1CNT;
  //  
  // Check if timer interrupt pending ...
  // 
  if (TPM1C0SC & TPM1C0SC_CH0F) {  // Adjust result
    Cnt = TPM1CNT;
    Time++;
  }
  return (OS_TIMER_RELOAD * Time) + Cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us
*
*       Convert Cycles into micro seconds.
*
*       If your clock frequency is not a multiple of 1 MHz,
*       you may have to modify this routine in order to get proper
*       diagonstics.
*
*       This routine is required for profiling or high resolution time
*       measurement only. It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_PCLK_TIMER/1000000);
}

/*********************************************************************
*
*       Communication for OSView via UART (optional)
*
**********************************************************************
*/

#if OS_UART_USED

#define BAUDDIVIDE (OS_PCLK_UART / (16 * OS_BAUDRATE))

/*********************************************************************
*
*       OS_COM_ISR_Rx() 
*
*       OS USART Rx interrupt handler 
*/
__interrupt void OS_COM_ISR_Rx(void) {
  OS_U8 c;
  OS_EnterInterrupt();
  c = SCIxD;
  if (((SCIxS1 & SCIxS1_ERRORS_MASK) == 0) && (SCIxS1 & SCIxS1_RDRF)) {  // No Error occured and Rx buffer full?
    OS_OnRx(c);
  }
  OS_LeaveInterrupt();
}

/*********************************************************************
*
*       OS_COM_ISR_Tx() 
*
*       OS USART Tx interrupt handler 
*/
__interrupt void OS_COM_ISR_Tx(void) {
  OS_EnterInterrupt();
  if (OS_OnTx()) {             // No more characters to send?  
    SCIxC2 &= ~SCIxC2_TIE;     // Disable Transmission Complete Interrupt Enable (for TC)     
  }
  OS_LeaveInterrupt();
}

/*********************************************************************
*
*       OS_COM_Send1()
*       Never call this function directly from your application
*/
void OS_COM_Send1(OS_U8 c) {
  while ((SCIxS1 & SCIxS1_TDRE) == 0);  // Is transmit buffer empty?
  SCIxC2 |= SCIxC2_TIE; 
  SCIxD   = c;  
}

/*********************************************************************
*
*       OS_COM_Init()
*       Initialize UART for OSView
*/
void OS_COM_Init(void) {       // Initialize UART, enable UART interrupts 
  SCIxC1  = 0x00;              // SCI Control Register 1
  SCIxBDH = BAUDDIVIDE >> 8;   // Set baudrate
  SCIxBDL = BAUDDIVIDE & 0xFF;
  SCIxC2  =   SCIxC2_RIE       // Receive Complete Interrupt Enable (for TC)
            | SCIxC2_TE        // Transmitter Enable
            | SCIxC2_RE;       // Receiver Enable
}

#else 

void OS_COM_Init(void) {}
void OS_COM_Send1(OS_U8 c) {
  (void) c;                    // Avoid compiler warning 
  OS_COM_ClearTxActive();      // Let the OS know that Tx is not busy 
}

#endif /*  OS_UART_USED  */

/****** Final check of configuration ********************************/

#ifndef OS_UART_USED
  #error "OS_UART_USED has to be defined"
#endif

/*********************************************************************
*
*       ISR specific function and interrupt vector table
*
**********************************************************************
*/

/*********************************************************************
*
*       _OS_DefaultInterrupt()
*
*       This is a default interrupt, which is called when no specific 
*       interrupt handler is listed in the interrupt vector table
* 
*/
__interrupt static void _OS_DefaultInterrupt(void) {
  while (1);  
}

/*********************************************************************
*
*       Interrupt Vector Table
*
* 
*/
void (* near const _vect[])() @0xFFCC = { /* Interrupt vector table */
         _OS_DefaultInterrupt,            /* Int.no. 25 Vrti       (at FFCC)    Unassigned   */
         _OS_DefaultInterrupt,            /* Int.no. 24 Viic1      (at FFCE)    Unassigned   */
         _OS_DefaultInterrupt,            /* Int.no. 23 Vatd1      (at FFD0)    Unassigned   */
         _OS_DefaultInterrupt,            /* Int.no. 22 Vkeyboard1 (at FFD2)    Unassigned   */
#if ((OS_UART_USED == 1) && (OS_UART == 1))
         OS_COM_ISR_Tx,                   /* Int.no. 21 Vsci2tx    (at FFD4)    Unassigned   */
         OS_COM_ISR_Rx,                   /* Int.no. 20 Vsci2rx    (at FFD6)    Unassigned   */
#else
         _OS_DefaultInterrupt,            /* Int.no. 21 Vsci2tx    (at FFD4)    Unassigned   */
         _OS_DefaultInterrupt,            /* Int.no. 20 Vsci2rx    (at FFD6)    Unassigned   */
#endif
         _OS_DefaultInterrupt,            /* Int.no. 19 Vsci2err   (at FFD8)    Unassigned   */
#if ((OS_UART_USED == 1) && (OS_UART == 0))
         OS_COM_ISR_Tx,                   /* Int.no. 18 Vsci1tx    (at FFDA)    Used         */
         OS_COM_ISR_Rx,                   /* Int.no. 17 Vsci1rx    (at FFDC)    Used         */
#else
         _OS_DefaultInterrupt,            /* Int.no. 18 Vsci1tx    (at FFDA)    Unassigned   */
         _OS_DefaultInterrupt,            /* Int.no. 17 Vsci1rx    (at FFDC)    Unassigned   */
#endif
         _OS_DefaultInterrupt,            /* Int.no. 16 Vsci1err   (at FFDE)    Unassigned   */
         _OS_DefaultInterrupt,            /* Int.no. 15 Vspi1      (at FFE0)    Unassigned   */
         _OS_DefaultInterrupt,            /* Int.no. 14 Vtpm2ovf   (at FFE2)    Unassigned   */
         _OS_DefaultInterrupt,            /* Int.no. 13 Vtpm2ch4   (at FFE4)    Unassigned   */
         _OS_DefaultInterrupt,            /* Int.no. 12 Vtpm2ch3   (at FFE6)    Unassigned   */
         _OS_DefaultInterrupt,            /* Int.no. 11 Vtpm2ch2   (at FFE8)    Unassigned   */
         _OS_DefaultInterrupt,            /* Int.no. 10 Vtpm2ch1   (at FFEA)    Unassigned   */
         _OS_DefaultInterrupt,            /* Int.no.  9 Vtpm2ch0   (at FFEC)    Unassigned   */
         _OS_DefaultInterrupt,            /* Int.no.  8 Vtpm1ovf   (at FFEE)    Unassigned   */
         _OS_DefaultInterrupt,            /* Int.no.  7 Vtpm1ch2   (at FFF0)    Unassigned   */
         _OS_DefaultInterrupt,            /* Int.no.  6 Vtpm1ch1   (at FFF2)    Unassigned   */
         _OS_SystemIrqhandler,            /* Int.no.  5 Vtpm1ch0   (at FFF4)    Used         */
         _OS_DefaultInterrupt,            /* Int.no.  4 Vicg       (at FFF6)    Unassigned   */
         _OS_DefaultInterrupt,            /* Int.no.  3 Vlvd       (at FFF8)    Unassigned   */
         _OS_DefaultInterrupt,            /* Int.no.  2 Virq       (at FFFA)    Unassigned   */
         _OS_DefaultInterrupt,            /* Int.no.  1 Vswi       (at FFFC)    Unassigned   */
};

/*****  End Of File *************************************************/

